==> aTunes_1.9.0.tar.gz: aTunes/src/net/sourceforge/atunes/kernel/modules/playlist/PlayList.java <==
/*
 * aTunes 1.9.0
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.playlist;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Random;

import net.sourceforge.atunes.kernel.handlers.PlayerHandler;
import net.sourceforge.atunes.kernel.modules.podcast.PodcastFeedEntry;
import net.sourceforge.atunes.kernel.modules.radio.Radio;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.utils.StringUtils;

/**
 * This class represents a play list.
 * 
 * @author fleax
 */
public class PlayList extends ArrayList<AudioObject> {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1793886828081842328L;

	/** Name of play list as shown on play list tabs. */
	private String name;

	/** Random generator for shuffle mode. */
	private Random randomGenerator;

	/** Counter for random mode, counts number of songs played in shuffle mode. */
	private transient int randomCounter = 1;

	/** A pointer to the current object to play. */
	private int currentAudioObject;

	/** A list containing previous played elements. */
	private ArrayList<AudioObject> previousPlayedList;

	/** Index of previous played elements. */
	private int previousPlayedIndex = 0;

	/**
	 * Default Constructor.
	 */
	public PlayList() {
		this(new ArrayList<AudioObject>());
	}

	/**
	 * Constructor.
	 * 
	 * @param files
	 *            the files
	 */
	public PlayList(List<AudioObject> files) {
		super(files);
		// Initialize random generator
		randomGenerator = new Random(System.currentTimeMillis());
		previousPlayedList = new ArrayList<AudioObject>();
	}

	/**
	 * Instantiates a new play list.
	 * 
	 * @param anotherPlayList
	 *            the another play list
	 */
	public PlayList(PlayList anotherPlayList) {
		super(anotherPlayList);
		randomGenerator = anotherPlayList.randomGenerator;
		randomCounter = anotherPlayList.randomCounter;
		currentAudioObject = anotherPlayList.currentAudioObject;
		name = anotherPlayList.name;
		previousPlayedIndex = anotherPlayList.previousPlayedIndex;
		previousPlayedList = anotherPlayList.previousPlayedList;
	}

	/**
	 * Clear play list.
	 */
	@Override
	public void clear() {
		previousPlayedList.clear();
		previousPlayedIndex = 0;
		super.clear();
	}

	/**
	 * Return current audio object.
	 * 
	 * @return the current audio object
	 */
	public AudioObject getCurrentAudioObject() {
		return size() > 0 && size() > currentAudioObject ? get(currentAudioObject) : null;
	}

	/**
	 * Returns play list length in string format.
	 * 
	 * @return the length
	 */
	public String getLength() {
		long seconds = 0;
		for (AudioObject song : this) {
			seconds += song.getDuration();
		}
		return StringUtils.fromSecondsToHoursAndDays(seconds);
	}

	/**
	 * Gets the name.
	 * 
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * Gets the index of the next file.
	 * 
	 * @return the current audio object index
	 */
	public int getCurrentAudioObjectIndex() {
		return currentAudioObject;
	}

	/**
	 * Returns the next file to play.
	 * 
	 * @return the next audio object to play
	 */
	public AudioObject getNextAudioObjectToPlay() {
		// If previousPlayedIndex == 0 means we have to calculate the next audio object to play
		if (previousPlayedIndex >= -1) {
			previousPlayedIndex = 0;
			if (previousPlayedList == null)
				previousPlayedList = new ArrayList<AudioObject>();
			if (PlayerHandler.getInstance().isShuffle()) {
				if (PlayerHandler.getInstance().isRepeat()) {
					randomCounter = 1;
					int next = getRandomIndex();
					currentAudioObject = next;
					AudioObject nextAO = get(currentAudioObject);
					previousPlayedList.add(nextAO);
					return nextAO;
				} else if (randomCounter < size()) {
					randomCounter++;
					// randomCounter gets reset to 0 upon song deletion, so if a user deletes down to one track, return a valid value.
					int next = 0;
					if (size() <= 1) {
						next = 0;
					} else {
						// If next track is the same than previous played, select previous or next track
						next = getRandomIndex();
						if (next == currentAudioObject) {
							if (next < size() - 1) {
								next++;
							} else {
								next--;
							}
						}
					}
					currentAudioObject = next;
					AudioObject nextAO = get(currentAudioObject);
					previousPlayedList.add(nextAO);
					return nextAO;
				} else {
					return null;
				}
			} else if (currentAudioObject < size() - 1) {
				currentAudioObject++;
				AudioObject nextAO = get(currentAudioObject);
				previousPlayedList.add(nextAO);
				return nextAO;
			} else {
				if (PlayerHandler.getInstance().isRepeat()) {
					currentAudioObject = 0;
					AudioObject nextAO = get(currentAudioObject);
					previousPlayedList.add(nextAO);
					return nextAO;
				}
				return null;
			}
		}

		// previousPlayedIndex != 0 (< 0) means user have pressed previous buttons a few times, so next audio object has to be
		// an already audio object
		previousPlayedIndex++;
		currentAudioObject = indexOf(previousPlayedList.get(previousPlayedList.size() + previousPlayedIndex));
		AudioObject nextAO = get(currentAudioObject);
		return nextAO;
	}

	/**
	 * Returns number of audio files in play list.
	 * 
	 * @return the number of audio files
	 */
	public int getNumberOfAudioFiles() {
		int result = 0;
		for (AudioObject audioObject : this) {
			if (audioObject instanceof AudioFile)
				result++;
		}
		return result;
	}

	/**
	 * Returns number of podcast feed entries in play list.
	 * 
	 * @return the number of podcast feed entries
	 */
	public int getNumberOfPodcastFeedEntries() {
		int result = 0;
		for (AudioObject audioObject : this) {
			if (audioObject instanceof PodcastFeedEntry)
				result++;
		}
		return result;
	}

	/**
	 * Returns number of radios in play list.
	 * 
	 * @return the number of radios
	 */
	public int getNumberOfRadios() {
		int result = 0;
		for (AudioObject audioObject : this) {
			if (audioObject instanceof Radio)
				result++;
		}
		return result;
	}

	/**
	 * Return the previous file played.
	 * 
	 * @return the previous audio object to play
	 */
	public AudioObject getPreviousAudioObjectToPlay() {
		// There is no previous played audio object to play again...
		if (previousPlayedList == null || previousPlayedList.isEmpty() || previousPlayedList.size() + previousPlayedIndex <= 0) {
			if (previousPlayedList == null)
				previousPlayedList = new ArrayList<AudioObject>();

			if (PlayerHandler.getInstance().isShuffle()) {
				if (randomCounter < size() || PlayerHandler.getInstance().isRepeat()) {
					randomCounter++;
					if (randomCounter == size())
						randomCounter = 0;
					int prevFile = getRandomIndex();
					currentAudioObject = prevFile;
					AudioObject prevAO = get(prevFile);
					previousPlayedList.add(0, prevAO);
					previousPlayedIndex--;
					return prevAO;
				}
				return null;
			} else if (currentAudioObject > 0) {
				AudioObject prevAO = get(--currentAudioObject);
				previousPlayedList.add(0, prevAO);
				previousPlayedIndex--;
				return prevAO;
			} else {
				if (PlayerHandler.getInstance().isRepeat()) {
					currentAudioObject = size() - 1;
					if (currentAudioObject == -1)
						return null;

					AudioObject prevAO = get(currentAudioObject);
					previousPlayedList.add(0, prevAO);
					previousPlayedIndex--;
					return prevAO;
				}
				return null;
			}
		}

		previousPlayedIndex--;
		// when pressing previous when previousPlayedIndex == 0, last audio object in previousPlayedList is current audio object
		// so we must add one more -1
		if (previousPlayedIndex == -1)
			previousPlayedIndex--;
		if (previousPlayedList.size() + previousPlayedIndex < 0) {
			currentAudioObject--;
			previousPlayedList.add(0, get(currentAudioObject));
		} else {
			currentAudioObject = indexOf(previousPlayedList.get(previousPlayedList.size() + previousPlayedIndex));
		}
		return get(currentAudioObject);
	}

	/**
	 * Returns a random position in playlist.
	 * 
	 * @return the random position
	 */
	public int getRandomPosition() {
		return randomGenerator.nextInt(size());
	}

	/**
	 * Removes a song.
	 * 
	 * @param index
	 *            the index
	 * 
	 * @return the audio object
	 */
	@Override
	public AudioObject remove(int index) {
		randomCounter = 0;
		AudioObject ao = get(index);
		ArrayList<AudioObject> list = new ArrayList<AudioObject>();
		list.add(ao);
		previousPlayedList.removeAll(list);
		if (previousPlayedList.isEmpty())
			previousPlayedIndex = 0;
		return super.remove(index);
	}

	/**
	 * Removes a list of songs.
	 * 
	 * @param c
	 *            the c
	 * 
	 * @return true, if removes the all
	 */
	@Override
	public boolean removeAll(Collection<?> c) {
		randomCounter = 0;
		previousPlayedList.removeAll(c);
		if (previousPlayedList.isEmpty())
			previousPlayedIndex = 0;
		return super.removeAll(c);
	}

	/**
	 * Sets the name.
	 * 
	 * @param name
	 *            the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Sets the index of the next file.
	 * 
	 * @param newCurrentAudioObject
	 *            the new current audio object
	 */
	public void setCurrentAudioObject(int newCurrentAudioObject) {
		if (isEmpty())
			return;
		AudioObject currentAO = null;
		if (size() > currentAudioObject)
			currentAO = get(currentAudioObject);
		this.currentAudioObject = newCurrentAudioObject;
		AudioObject newCurrentAO = get(currentAudioObject);
		if (currentAO == null || currentAO != newCurrentAO) {
			previousPlayedList.add(newCurrentAO);
		}
	}

	/**
	 * Sets a list of songs to this playlist.
	 * 
	 * @param songs
	 *            the songs
	 */
	public void setSongs(List<AudioObject> songs) {
		clear();
		addAll(songs);
	}

	/**
	 * Return a random index.
	 * 
	 * @return the random index
	 */
	private int getRandomIndex() {
		if (size() == 0)
			return 0;
		return randomGenerator.nextInt(size());
	}
}

==> aTunes_1.10.1.tar.gz: aTunes/src/net/sourceforge/atunes/kernel/modules/playlist/PlayList.java <==
/*
 * aTunes 1.10.1
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.playlist;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Random;

import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.kernel.modules.podcast.PodcastFeedEntry;
import net.sourceforge.atunes.kernel.modules.radio.Radio;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.utils.StringUtils;

/**
 * This class represents a play list.
 * 
 * @author fleax
 */
public class PlayList extends ArrayList<AudioObject> {

	/** The Constant serialVersionUID. */
	private static final long serialVersionUID = 1793886828081842328L;

	/** Name of play list as shown on play list tabs. */
	private String name;

	/** Random generator for shuffle mode. */
	private Random randomGenerator;

	/** Counter for random mode, counts number of songs played in shuffle mode. */
	private transient int randomCounter = 1;

	/** A pointer to the current object to play. */
	private int currentAudioObject;

	/** A list containing previous played elements. */
	private ArrayList<AudioObject> previousPlayedList;

	/** Index of previous played elements. */
	private int previousPlayedIndex = 0;

	/**
	 * Default Constructor.
	 */
	public PlayList() {
		this(new ArrayList<AudioObject>());
	}

	/**
	 * Constructor.
	 * 
	 * @param files
	 *            the files
	 */
	public PlayList(List<AudioObject> files) {
		super(files);
		// Initialize random generator
		randomGenerator = new Random(System.currentTimeMillis());
		previousPlayedList = new ArrayList<AudioObject>();
	}

	/**
	 * Instantiates a new play list.
	 * 
	 * @param anotherPlayList
	 *            the another play list
	 */
	public PlayList(PlayList anotherPlayList) {
		super(anotherPlayList);
		randomGenerator = anotherPlayList.randomGenerator;
		randomCounter = anotherPlayList.randomCounter;
		currentAudioObject = anotherPlayList.currentAudioObject;
		name = anotherPlayList.name;
		previousPlayedIndex = anotherPlayList.previousPlayedIndex;
		previousPlayedList = anotherPlayList.previousPlayedList;
	}

	/**
	 * Clear play list.
	 */
	@Override
	public void clear() {
		previousPlayedList.clear();
		previousPlayedIndex = 0;
		super.clear();
	}

	/**
	 * Return current audio object.
	 * 
	 * @return the current audio object
	 */
	public AudioObject getCurrentAudioObject() {
		return size() > 0 && size() > currentAudioObject ? get(currentAudioObject) : null;
	}

	/**
	 * Returns play list length in string format.
	 * 
	 * @return the length
	 */
	public String getLength() {
		long seconds = 0;
		for (AudioObject song : this) {
			seconds += song.getDuration();
		}
		return StringUtils.fromSecondsToHoursAndDays(seconds);
	}

	/**
	 * Gets the name.
	 * 
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * Gets the index of the next file.
	 * 
	 * @return the current audio object index
	 */
	public int getCurrentAudioObjectIndex() {
		return currentAudioObject;
	}

	/**
	 * Returns the next file to play.
	 * 
	 * @return the next audio object to play
	 */
	public AudioObject getNextAudioObjectToPlay() {
		// If previousPlayedIndex == 0 means we have to calculate the next audio object to play
		if (previousPlayedIndex >= -1) {
			previousPlayedIndex = 0;
			if (previousPlayedList == null) {
				previousPlayedList = new ArrayList<AudioObject>();
			}
			if (Kernel.getInstance().state.isShuffle()) {
				if (Kernel.getInstance().state.isRepeat()) {
					randomCounter = 1;
					int next = getRandomIndex();
					currentAudioObject = next;
					AudioObject nextAO = get(currentAudioObject);
					previousPlayedList.add(nextAO);
					return nextAO;
				} else if (randomCounter < size()) {
					randomCounter++;
					// randomCounter gets reset to 0 upon song deletion, so if a user deletes down to one track, return a valid value.
					int next = 0;
					if (size() <= 1) {
						next = 0;
					} else {
						// If next track is the same than previous played, select previous or next track
						next = getRandomIndex();
						if (next == currentAudioObject) {
							if (next < size() - 1) {
								next++;
							} else {
								next--;
							}
						}
					}
					currentAudioObject = next;
					AudioObject nextAO = get(currentAudioObject);
					previousPlayedList.add(nextAO);
					return nextAO;
				} else {
					return null;
				}
			} else if (currentAudioObject < size() - 1) {
				currentAudioObject++;
				AudioObject nextAO = get(currentAudioObject);
				previousPlayedList.add(nextAO);
				return nextAO;
			} else {
				if (Kernel.getInstance().state.isRepeat()) {
					currentAudioObject = 0;
					AudioObject nextAO = get(currentAudioObject);
					previousPlayedList.add(nextAO);
					return nextAO;
				}
				return null;
			}
		}

		// previousPlayedIndex != 0 (< 0) means user have pressed previous buttons a few times, so next audio object has to be
		// an already audio object
		previousPlayedIndex++;
		currentAudioObject = indexOf(previousPlayedList.get(previousPlayedList.size() + previousPlayedIndex));
		AudioObject nextAO = get(currentAudioObject);
		return nextAO;
	}

	/**
	 * Returns number of audio files in play list.
	 * 
	 * @return the number of audio files
	 */
	public int getNumberOfAudioFiles() {
		int result = 0;
		for (AudioObject audioObject : this) {
			if (audioObject instanceof AudioFile) {
				result++;
			}
		}
		return result;
	}

	/**
	 * Returns number of podcast feed entries in play list.
	 * 
	 * @return the number of podcast feed entries
	 */
	public int getNumberOfPodcastFeedEntries() {
		int result = 0;
		for (AudioObject audioObject : this) {
			if (audioObject instanceof PodcastFeedEntry) {
				result++;
			}
		}
		return result;
	}

	/**
	 * Returns number of radios in play list.
	 * 
	 * @return the number of radios
	 */
	public int getNumberOfRadios() {
		int result = 0;
		for (AudioObject audioObject : this) {
			if (audioObject instanceof Radio) {
				result++;
			}
		}
		return result;
	}

	/**
	 * Return the previous file played.
	 * 
	 * @return the previous audio object to play
	 */
	public AudioObject getPreviousAudioObjectToPlay() {
		// There is no previous played audio object to play again...
		if (previousPlayedList == null || previousPlayedList.isEmpty() || previousPlayedList.size() + previousPlayedIndex <= 0) {
			if (previousPlayedList == null) {
				previousPlayedList = new ArrayList<AudioObject>();
			}

			if (Kernel.getInstance().state.isShuffle()) {
				if (randomCounter < size() || Kernel.getInstance().state.isRepeat()) {
					randomCounter++;
					if (randomCounter == size()) {
						randomCounter = 0;
					}
					int prevFile = getRandomIndex();
					currentAudioObject = prevFile;
					AudioObject prevAO = get(prevFile);
					previousPlayedList.add(0, prevAO);
					previousPlayedIndex--;
					return prevAO;
				}
				return null;
			} else if (currentAudioObject > 0) {
				AudioObject prevAO = get(--currentAudioObject);
				previousPlayedList.add(0, prevAO);
				previousPlayedIndex--;
				return prevAO;
			} else {
				if (Kernel.getInstance().state.isRepeat()) {
					currentAudioObject = size() - 1;
					if (currentAudioObject == -1) {
						return null;
					}

					AudioObject prevAO = get(currentAudioObject);
					previousPlayedList.add(0, prevAO);
					previousPlayedIndex--;
					return prevAO;
				}
				return null;
			}
		}

		previousPlayedIndex--;
		// when pressing previous when previousPlayedIndex == 0, last audio object in previousPlayedList is current audio object
		// so we must add one more -1
		if (previousPlayedIndex == -1) {
			previousPlayedIndex--;
		}
		if (previousPlayedList.size() + previousPlayedIndex < 0) {
			currentAudioObject--;
			previousPlayedList.add(0, get(currentAudioObject));
		} else {
			currentAudioObject = indexOf(previousPlayedList.get(previousPlayedList.size() + previousPlayedIndex));
		}
		return get(currentAudioObject);
	}

	/**
	 * Returns a random position in playlist.
	 * 
	 * @return the random position
	 */
	public int getRandomPosition() {
		return randomGenerator.nextInt(size());
	}

	/**
	 * Removes a song.
	 * 
	 * @param index
	 *            the index
	 * 
	 * @return the audio object
	 */
	@Override
	public AudioObject remove(int index) {
		randomCounter = 0;
		AudioObject ao = get(index);
		ArrayList<AudioObject> list = new ArrayList<AudioObject>();
		list.add(ao);
		previousPlayedList.removeAll(list);
		if (previousPlayedList.isEmpty()) {
			previousPlayedIndex = 0;
		}
		return super.remove(index);
	}

	/**
	 * Removes a list of songs.
	 * 
	 * @param c
	 *            the c
	 * 
	 * @return true, if removes the all
	 */
	@Override
	public boolean removeAll(Collection<?> c) {
		randomCounter = 0;
		previousPlayedList.removeAll(c);
		if (previousPlayedList.isEmpty()) {
			previousPlayedIndex = 0;
		}
		return super.removeAll(c);
	}

	/**
	 * Sets the name.
	 * 
	 * @param name
	 *            the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * Sets the index of the next file.
	 * 
	 * @param newCurrentAudioObject
	 *            the new current audio object
	 */
	public void setCurrentAudioObject(int newCurrentAudioObject) {
		if (isEmpty()) {
			return;
		}
		AudioObject currentAO = null;
		if (size() > currentAudioObject) {
			currentAO = get(currentAudioObject);
		}
		this.currentAudioObject = newCurrentAudioObject;
		AudioObject newCurrentAO = get(currentAudioObject);
		if (currentAO == null || currentAO != newCurrentAO) {
			previousPlayedList.add(newCurrentAO);
		}
	}

	/**
	 * Sets a list of songs to this playlist.
	 * 
	 * @param songs
	 *            the songs
	 */
	public void setSongs(List<AudioObject> songs) {
		clear();
		addAll(songs);
	}

	/**
	 * Return a random index.
	 * 
	 * @return the random index
	 */
	private int getRandomIndex() {
		if (size() == 0) {
			return 0;
		}
		return randomGenerator.nextInt(size());
	}
}
